decode_XOR_charlist64(){
	# only read for first block of iphrArray if multiple used for key
	if (( ! isSubsequentIPHR )); then
		read_b64_XOR_key_input || return 1
	fi

	#(( passphraseLength > 23 )) && p_err "longer passphrases than 23 chars not implemented yet" && return 1
	
	XORed_c64_encoded_arr+=( ${hexadesIPHR[@]:1:passphraseLength} )
	debugMsg "hexadesIPHR:" "$(printf "%02x " "${hexadesIPHR[@]}")"   \
	         "XORed_c64_encoded_arr:" "$(printf "%02x " "${XORed_c64_encoded_arr[@]}")"

	# if sbsqtKID, return to calling decodeIPHR() to get next iphrArray element
	if (( sbsqtKID && ! directDecode )); then
		return 0
	fi
	
	XORed_c64_encoded_arr=( ${XORed_c64_encoded_arr[@]:0:${#XOR_key_input[@]}} )
	
	# if inserted XOR key length is bigger blocklength, obfuscate output
	#   of 'xor_two_arrays()' to not reveal the length (block counts) in error message.
	#   By appending part of hexadesIPHR again.
	#   (not really relevant, since how long the key is in block sizes (23 hexades per
	#   block) can be seen in settings hexade.)
	if (( ${#XORed_c64_encoded_arr[@]} < passphraseLength )); then
		append_len=$(( passphraseLength - ${#XORed_c64_encoded_arr[@]} ))
		XORed_c64_encoded_arr+=( ${hexadesIPHR[@]:0:append_len} )
	fi
	
	xor_two_arrays XORed_c64_encoded_arr XOR_key_input || return 1

	decode_c64 XORed_arr || return 1
	
	XORed_c64_encoded_arr=() # empty arr for next use in interactive mode
	b64_XOR_key_input=""
	
	return 0
}

# read base64 encoded key to decrypt iphr
read_b64_XOR_key_input(){
	local answer
	checksum_len=3 # crc 16 is 2 byte = 3 base64 chars (+ 1 '=')
	printf "Provide the keys XOR string to decode (grouping by spaces allowed):\n"
	IFS= read -r b64_XOR_key_input < /dev/tty # read -r: Backslash is not escaping.
	(( !debug )) && printf "\033[1A\033[2K"
	echo
	while (( ${#b64_XOR_key_input} < (checksum_len+1) )); do
		printf "No input with crc checksum given. Try again? (y/n, or type 'd' to deactivate checksum): "
		read -n 1 answer ; printf "\033[2K\r"
		[[ "$answer" != [yd] ]] && return 1
		debugMsg "not returning"
		[[ "$answer" == "d" ]] && checksum_len=0 && break
		IFS= read -r b64_XOR_key_input < /dev/tty # read -r: Backslash is not escaping.
		echo
	done
	# I want do see the input to check, don't have to type it twice, and want to
	# remove it from screen afterwards. So:
	# delete one line above (the input XOR key)
	(( !debug )) && printf "\033[2A\033[2K"

	# remove spaces from input
	b64_XOR_key_input=$(echo "$b64_XOR_key_input" | tr -d ' ')
	
	# separate crc16 and data (append 2 '=' to crc for base64 decoding)
	crc16_in="${b64_XOR_key_input: -checksum_len}="
	(( checksum_len )) && b64_XOR_key_input_chk=${b64_XOR_key_input:0: -checksum_len}

	# check crc16
	crc16_chck="$(
		$crc16_nk $(printf "%s" ${b64_XOR_key_input_chk})  \
			| xxd -r -p  \
			| base64
	)"

	debugMsg "b64_XOR_key_input" "$(printf "%s" ${b64_XOR_key_input})"  \
	         "crc16_in:   $crc16_in"       \
	         "crc16_chck: $crc16_chck"

	while [[ "$crc16_in" != "$crc16_chck" ]] && (( checksum_len )); do
		echo "The checksum of your input doesn't match. Try again? (y/n or type 'd' to deactivate checksum) "
		read -n 1 answer ; printf "\033[2K\r"
		if [[ "$answer" == "y" ]]; then
			read_b64_XOR_key_input && return 0 || return 1
		elif [[ "$answer" == "d" ]]; then
			checksum_len=0
		else
			return 1
		fi
	done
	(( checksum_len )) && b64_XOR_key_input="$b64_XOR_key_input_chk"

	passphraseLength=${#b64_XOR_key_input}
	XOR_key_input=()
	local i
	for((i=0; i<passphraseLength; i++)); do
		idx=0
		while [[ "${b64_XOR_key_input:i:1}" != "${base64_charset[idx]}" ]]; do
			((idx++))
			(( idx > 63 )) && p_err "XOR string was not base64" && return 1
		done
		XOR_key_input[i]=$idx
	done
	return 0
}


decoded_c64_string=""
decode_c64(){ # argument $1 should be the array name to decode
	[[ -z $1 ]] \
		&& p_err "argument missing." \
		         "usage: $FUNCNAME <input_arra_name>" "result in '\$decoded_c64_string'"\
		&& return 1
	local -n input_arr=$1 # pass an array to a function by name reference
	                      # https://stackoverflow.com/questions/16461656/how-to-pass-array-as-an-argument-to-a-function-in-bash
	debugMsg "input_arr ('$1' len: ${#input_arr[@]}):" "$(printf "%02o " "${input_arr[@]}")" "$(declare -p XORed_arr)"
	decoded_c64_string=""
	local i
	for((i=0; i<${#input_arr[@]}; i++ )); do
		if (( ${input_arr[i]}<62 )); then
		# use charlist64_1
			(( ${input_arr[i]} == 0 )) && break
			decoded_c64_string+=${charlist64_1[${input_arr[i]}]}
		elif (( ${input_arr[i]}==62 )); then
		# use charlist64_2
			((i++))
			(( i>=${#input_arr[@]} )) && break
			decoded_c64_string+=${charlist64_2[${input_arr[i]}]}
		elif (( ${input_arr[i]}==63 )); then
		# utf8 decoding
			((i++))
			decode_utf8_hexades $i || return 1
		else
			p_err "wrong encoding: '${input_arr[i]}'"
		fi
	done
	return 0
}
decode_utf8_hexades(){
	[[ -z $1 ]] && p_err "index argument missing" && return 1
	i=$1 # input_arr index counter

	# init utf8 with startbyte (11bb bbbb)
	hex_utf8="$(printf "%02x" $(( ${input_arr[i]} | 0xc0 )) )"
	start_bits=$(( ( ( ${input_arr[i]} | 0xc0 ) << 1 ) & 0xff )) # shift off first bit for starting byte
	((i++))
	while (( start_bits & 0x80 )); do
		hex_utf8+="$(printf "%02x" $(( ${input_arr[i]} | 0x80 )) )"
		((i++))
		start_bits=$(( ($start_bits << 1) & 0xff ))
	done
	((i--))

	debugMsg "hex_utf8: $hex_utf8"
	if (( 0x$hex_utf8 < 0xc200 ));then
		decode_invalid_utf8hexades_toASCII || return 1
		return 0
	fi

	decoded_c64_string+="$(printf "$hex_utf8" | xxd -p -r )"
	return 0
	#######
	# utf8 encoding/decoding explanation: see encoding func.
	#######
}
decode_invalid_utf8hexades_toASCII(){
	hex_ASCII=$(( ( 0x$hex_utf8 >> 2 ) & 0xc0)) # get higher 2 ASCII bits
	debugMsg "hex_ASCII '$hex_ASCII'"
	hex_ASCII=$(( hex_ASCII | ( 0x$hex_utf8 & 0x3f ) )) # get lower 6 ASCII bits
	hex_ASCII="$(printf "%02x" $hex_ASCII)"
	debugMsg "hex_ASCII '$hex_ASCII'"
	if (( 0x$hex_ASCII == 0x0a )); then
		decoded_c64_string+=$'\n'
	else
		decoded_c64_string+="$(printf "$hex_ASCII" | xxd -p -r )"
	fi
	debugMsg "decoded_c64_string '$decoded_c64_string' $(printf "$decoded_c64_string" | xxd)"
	return 0
}

XORed_arr=()
xor_two_arrays(){ # args: $1 name of encoded array ; $2 name of XOR array
	[ -z $2 ] && p_err "no arg" && return 1
	XORed_arr=()
	local -n arg1_encoded_arr=$1 # pass array by name reference
	local -n arg2_XORkey=$2
	len=${#arg1_encoded_arr[@]}
	xorLen=${#arg2_XORkey[@]}
	(( len != xorLen ))  \
		&& p_err "encoded_string length doesn't match arg2_XORkey length ($len != $xorLen)"   \
		&& return 1
	local i
	for ((i=0; i<len; i++)); do
		(( XORed_arr[i] = arg1_encoded_arr[i] ^ arg2_XORkey[i] ))
	done
	# debugMsg ""                                       \
	# 	"arg1_encoded_arr ($1):"                      \
	# 	"$(printf "%02x " "${arg1_encoded_arr[@]}")"  \
	# 	"arg2_XORkey ($2)"                            \
	# 	"$(printf "%02x " "${arg2_XORkey[@]}")"       \
	# 	"XORed_arr"                               \
	# 	"$(printf "%02x " "${XORed_arr[@]}")"

	return 0
}
