# Date <-> Hex conversion functions
#
#    encodeHexDate()
#    decodeHexDate()
# 
# dateHex is a two Byte hex number
# encodes/decodes date (a day) to/from 4 hex chars from/to 'dd.mm.yyyy' and
# assigns to 'dateHex' / 'decodedDate'

dateHex=""
decodedDate=""

# Timezone defined as UTC for encoding/decoding. So set it
export TZ=UTC

# Short description
#
# Get bytes 3 and 4 of UTC unix time of a day by:
#
#    1. get seconds since epoch of 00:00:00 of this day
#          0x386be3f0 for 31 Dez 1999
#    2. chop off lower two bytes (devide by 0x10000)
#          0x386b
#    3. add 1 and securely be within queried date when
#       reverse the process (muliply by 0x10000).
#          0x386c
#       (lower two bytes were max 0xffff. On reversing we
#       are at 00:00:01 of queried day. Minimum would
#       be 0x0000, on reverse, we are 65536 (0x10000)
#       seconds after midnight: 18:12:16)
#
# Range:
# Implementation is for years without giving century:
# So range is from 0x0000 (1 Jan 1970) to 0xba36 (31.12.2068)
# Which are dateHex values 0x001 and 0xba37 respectively.
# Theoretically valid for dates within unsigned 32 bit int unix time.
# Up to 0xfffe5380 (6. Feb 2106 00:00:00) wich would be dateHex 0xffff

encodeHexDate(){
	input_date="$1"
	if [[ "$OSTYPE" == "darwin"* ]] ; then # mac os
		date_epoch=$(date -j -f 	\
			"%d.%m.%y %H%M%S"	\
			"$input_date 000000" 	\
			"+%s"			\
		2>&1) # collect errors with 'date_epoch' variable
	else # GNU / Linux
		date_epoch=$(date -d "$input_date 00:00:00" +"%s")
	fi
	# date_epoch is empty if format was wrong
	[[ "$date_epoch" != +([0-9]) ]] && p_err "Wrong format: '$1' (see help)" && return 1
	(( $date_epoch < 0 )) && p_err "Conversion range from 1.1.70 to 31.12.68 '$1'" && return 1
	dateHex=$(printf "%04x\n" $(( date_epoch / 0x10000 + 0x1 )) )
	return 0
}

decodeHexDate(){
	dateHex=$1
	# check format
	if ! checkHex $dateHex || ! checkArgLen $dateHex 4; then
		p_err "wrong format: '$1'"
		return 2
	fi
	if [[ "$OSTYPE" == "darwin"* ]] ; then # mac os
		decodedDate=$(date -j -f "%s" "$(( 0x$dateHex * 0x10000 ))" "+%d.%m.%Y")
	else # GNU / Linux
		decodedDate=$(date -d "$(( 0x$dateHex * 0x10000 ))" +"%s")
	fi
	[[ "$decodedDate" == "" ]] && p_err "date conversion error." && return 1
	return 0
}
